package com.alcorlink.smartcard;


import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import amlib.ccid.Reader;
import amlib.ccid.ReaderAT24C;
import amlib.ccid.ReaderException;
import amlib.ccid.SCError;
import amlib.debug.MyLog;

import amlib.hw.HardwareInterface;

public class AT24C_Activity extends ReaderActivity {

    private String mStrMessage;
    private ReaderAT24C m24C;


    private Button mBtOpen;
    private Button mBtClose;
    private Button mBtPower;
    private Button mBtWrite;
    private Button mBtRead;


    private EditText mEdtWriteAddr;
    private EditText mEdtWriteData;
    private EditText mEdtReadAddr;
    private EditText mEdtPageSize;
    private EditText mEdtReadLen;

    private byte mCardType;


    private static final String TAG = MainActivity.TAG;
    private static final String ACTION_USB_PERMISSION = "com.alcorlink.smartcard.USB_PERMISSION";
    private static final int MAX_WRITE_LENGTH = 128*1024;
    private static final byte CARD_TYPE_SINGLE = 0;
    private static final byte CARD_TYPE_DOUBLE = 1;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        setContentView(R.layout.activity_at24c);
        setupViews();
        super.onCreate(savedInstanceState);
    }


    public void OnClickAT24COpen (View view){
        requestDevPerm();
    }

    public void OnClickAT24CClose (View view){
        new CloseTask(this).execute();
    }

    public void OnClickAT24CPower(View view){
        mPowerDialog.show();
    }



    public void OnClickAT24CWrite(View view){
        int []pAddr = new int[1];
        byte []pPageSize = new byte[1];
        byte []pWrite = new byte[MAX_WRITE_LENGTH];
        int []pIntReturnLen = new int[1];

        //get data
        int result;
        //get addr
        result = getAddress(mEdtWriteAddr, pAddr);
        if (result != 0)
        {
            return;
        }
        //get page size
        result = getEditTextHex(mEdtPageSize, pPageSize, 2,pIntReturnLen);
        if (result != 0)
        {
            return;
        }
        //get data
        result = getEditTextHex(mEdtWriteData, pWrite, MAX_WRITE_LENGTH, pIntReturnLen);
        if (result != 0)
        {
            return;
        }
        Log.d(TAG, "pAddress="+ Integer.toHexString(pAddr[0]));
        Log.d(TAG, "pPageSize="+ Integer.toHexString(pPageSize[0]));
        result = m24C.AT24CSeriesCmd_Write(pAddr[0], mCardType, pPageSize[0], pIntReturnLen[0], pWrite);
        showResult("Write", result);
    }

    public void OnClickAT24CRead(View view){
        int []pAddr = new int[1];
        int []pLen = new int[1];
        byte []pRxBuf = new byte[MAX_WRITE_LENGTH];
        int []pIntReturnLen = new int[1];
        int result;
        //get addr
        result = getAddress(mEdtReadAddr, pAddr);
        if (result != 0)
        {
            return;
        }
        //get Length, 2 bytes
        result = getAddress(mEdtReadLen, pLen );
        if (result != 0)
        {
            return;
        }
        Log.d(TAG, "pAddress="+ Integer.toHexString(pAddr[0]));
        Log.d(TAG, "pLen="+ Integer.toHexString(pLen[0]));
        result = m24C.AT24CSeriesCmd_Read(pAddr[0], mCardType, pLen[0], pRxBuf, pIntReturnLen);
        if (result == 0)
        {
            Log.d(TAG,"result, pIntReturnLen="+pIntReturnLen[0]);
            //shows data
            showBufResult(pIntReturnLen[0], pRxBuf);
        }
        else
            showResult("Read", result);
    }




    private int getAddress(EditText edt, int []pAddress) {
        byte []pBuf = new byte[4];
        int []pIntReturnLen = new int[1];

        int result;
        result = getEditTextHex(edt, pBuf, 6, pIntReturnLen);//MAX 3 bytes
        if (result != 0)
        {
            return -1;
        }

        for (int i=(pIntReturnLen[0]-1);i>=0;i--)
        {
            pAddress[0] |= pBuf[i]<< (8*(pIntReturnLen[0]-1-i));
        }
        return 0;
    }


    /*
     * length: number of max digits in this field
     * */
    private int getEditTextHex(EditText edt, byte []pBuf, int length, int []pReturnLen)
    {
        byte []pByteArrary;
        String strText;
        int len;
        strText = edt.getText().toString();
        len = strText.length() ;
        Log.d(TAG, "get length="+len);
        if (len > length || len == 0 || (len%2 !=0))
        {
            mStrMessage = "Wrong length in text Fields ";
            Log.e(TAG, mStrMessage);
            mTextViewResult.setText(mStrMessage);
            return -1;
        }
        pByteArrary = MainActivity.toByteArray(strText);
        Log.d(TAG, "pByteArrary length="+pByteArrary.length);
        System.arraycopy(pByteArrary, 0, pBuf, 0, pByteArrary.length);
        pReturnLen[0] = pByteArrary.length;
        Log.d(TAG, "pBuf[0]=0x"+Integer.toHexString(pBuf[0]));
        return 0;
    }



    @Override
    public Reader readerSpecificInit() throws ReaderException {
        int status;
        Log.d(TAG,"readerSpecificInit");

        m24C = new ReaderAT24C((HardwareInterface) mMyDev);
        status = m24C.open();
        if (status != SCError.READER_SUCCESSFUL) {
            String s = "open AT24C fail:" + SCError.errorCode2String(status);
            MyLog.e(s);
            throw new ReaderException(s);
        }
        return m24C;
    }

    private RadioGroup.OnCheckedChangeListener listener = new RadioGroup.OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(RadioGroup group, int checkedId) {
            switch (checkedId) {
                case R.id.radioSingle:
                    Log.d(TAG, "select Single");
                    mCardType = CARD_TYPE_SINGLE;
                    break;
                case R.id.radioDouble:
                    Log.d(TAG, "select duble");
                    mCardType = CARD_TYPE_DOUBLE;
                    break;
            }
        }
    };


    @Override
    public void onCreateButtonSetup(){
        MyLog.enter();
        if (mUsbDev != null)
            mBtOpen.setEnabled(true);
        else
            mBtOpen.setEnabled(false);
        mBtClose.setEnabled(false);
        mBtPower.setEnabled(false);
        mBtWrite.setEnabled(false);
        mBtRead.setEnabled(false);
    }


    @Override
    public void onOpenButtonSetup( ){
       MyLog.enter();
        mBtOpen.setEnabled(false);
        mBtClose.setEnabled(true);
        mBtPower.setEnabled(true);
        mBtWrite.setEnabled(true);
        mBtRead.setEnabled(true);
    }



    @Override
    public void setButtonView()
    {
        mBtOpen = (Button)findViewById(R.id.AT24CbuttonOpen);
        mBtClose = (Button)findViewById(R.id.AT24CbuttonClose);
        mBtPower = (Button)findViewById(R.id.AT24Cbuttonpower);
        mBtWrite = (Button)findViewById(R.id.AT24CbuttonWrite);
        mBtRead = (Button)findViewById(R.id.AT24CbuttonRead);

    }

    @Override
    public void setEditView()
    {
        mEdtWriteAddr = (EditText)findViewById(R.id.AT24CEditTextWriteAddr);
        mEdtWriteData = (EditText)findViewById(R.id.AT24CEditTexData);
        mEdtReadAddr = (EditText)findViewById(R.id.AT24CEditTextReadAddr);
        mEdtPageSize = (EditText)findViewById(R.id.AT24CEditTexPageSize);
        mEdtReadLen = (EditText)findViewById(R.id.AT24CEditTexLen);
    }



    public void setRadioButtonView()
    {
        mCardType = CARD_TYPE_SINGLE;
        RadioGroup radioGroup = (RadioGroup) findViewById(R.id.radioGroup1);
        radioGroup.setOnCheckedChangeListener( listener);
        RadioButton bt = (RadioButton) findViewById(R.id.radioSingle);
        bt.setChecked(true);
    }


    @Override
    public void setTextView()
    {
        mTextViewResult = (TextView)findViewById(R.id.AT24CTextResult);
    }

    @Override
    public void setupViews() {
        MyLog.d("AT24C setupViews");
        setRadioButtonView();
        super.setupViews();
    }

}
