package com.alcorlink.smartcard;


import android.app.AlertDialog.Builder;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;

import java.lang.ref.WeakReference;

import amlib.ccid.Reader;
import amlib.ccid.ReaderAT88SC153;
import amlib.ccid.ReaderException;
import amlib.ccid.SCError;
import amlib.hw.HWType;
import amlib.hw.HardwareInterface;
import amlib.hw.HardwareInterfaceUsb;
import amlib.hw.ReaderHwException;

public class AT88SC153_Activity extends ReaderActivity {

	private String mStrMessage;
	private ReaderAT88SC153 m88sc153;

	private Button mBtOpen;
	private Button mBtClose;
	private Button mBtPower;
	private Button mBtWriteEeprom;
	private Button mBtReadEeprom;
	private Button mBtVerifyPassWd;

	private RadioGroup mRadioGroup;


	private EditText mEdtAddrEEPRead;
	private EditText mEdtLenEEPRead;
	private EditText mEdtAddrEEPWrite;
	private EditText mEdtDataEEPWrite;
	private EditText mEdtVerifyPw1;
	private EditText mEdtVerifyPw2;
	private EditText mEdtVerifyPw3;


	private Spinner mZoneSpinner;
	private Spinner mIndexSpinner;


	private static final String TAG = MainActivity.TAG;
	private static final boolean VERIFY_TYPE_WRITE = false;
	private static final boolean VERIFY_TYPE_READ = true;
	private static final int MAX_WRITE_LENGTH = 256;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		setContentView(R.layout.activity_at88sc153);
		setupViews();
		super.onCreate(savedInstanceState);
	}


	public void OnClickAT88SC153Open (View view){
		requestDevPerm();
	}

	public void OnClickAT88SC153Close (View view){
		new CloseTask(AT88SC153_Activity.this).execute();
	}

	public void OnClickAT88SC153Power(View view){
		mPowerDialog.show();
	}


	public void OnClickAT88SC153WriteEeprom(View view){
		byte []pAddr = new byte[1];
		byte []pWrite = new byte[MAX_WRITE_LENGTH];
		int []pIntReturnLen = new int[1];

		int result;
		byte bZone;
		bZone = (byte) getSpinnerSelect(mZoneSpinner);
		//get addr
		result = getEditTextHex(mEdtAddrEEPWrite, pAddr, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		//get data
		result = getEditTextHex(mEdtDataEEPWrite, pWrite, MAX_WRITE_LENGTH*2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}


		byte writeLen = (byte)pIntReturnLen[0];
		result = m88sc153.AT88SC153Cmd_WriteEeprom(
				bZone,
				pAddr[0],
				writeLen,
				pWrite );
		showResult("WriteEEPROM", result);
	}

	public void OnClickAT88SC153ReadEeprom(View view){
		byte []pAddr = new byte[1];
        byte []pReadLen = new byte[1];
		byte []pReadData = new byte[MAX_WRITE_LENGTH];
		int []pIntReturnLen = new int[1];

		Log.d(TAG, "");
		int result;
		byte bZone;
		bZone = (byte) getSpinnerSelect(mZoneSpinner);
		//get addr
		result = getEditTextHex(mEdtAddrEEPRead, pAddr, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		//get lenght
		result = getEditTextHex(mEdtLenEEPRead, pReadLen, 3,pIntReturnLen);
		if (result != 0)
		{
			return;
		}

		result = m88sc153. AT88SC153Cmd_ReadEeprom(
				bZone,
				pAddr[0],
				(byte)pReadLen[0],
				pReadData,
				pIntReturnLen);

		if (result == 0)
		{
			Log.d(TAG,"result, pIntReturnLen="+pIntReturnLen[0]);
			//shows data
			showBufResult(pIntReturnLen[0], pReadData);
		}
		else
			showResult("ReadEEPROM", result);
	}

	public void OnClickAT88SC153VerifyPassWd(View view){
		boolean function;
		byte bPwIndex = 0;
		int []pIntReturnLen = new int[1];
		int result ;
		byte []pPw1 = new byte[1];
		byte []pPw2 = new byte[1];
		byte []pPw3 = new byte[1];


		/*To ge tindex*/
		function = getRadioRWSelect();/*true for read*/
		bPwIndex = (byte) getSpinnerSelect(mIndexSpinner);
		result = getEditTextHex(mEdtVerifyPw1, pPw1, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = getEditTextHex(mEdtVerifyPw2, pPw2, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = getEditTextHex(mEdtVerifyPw3, pPw3, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		if (function)
			Log.d(TAG, "READ");
		else
			Log.d(TAG, "Write");
		Log.d(TAG, Integer.toHexString(0x000000ff&pPw1[0]) +"-"+ Integer.toHexString(0x000000ff&pPw2[0])+"-"+Integer.toHexString(0x000000ff&pPw3[0]) + "pw-"+bPwIndex);
		result = m88sc153.AT88SC153Cmd_VerifyPassword(bPwIndex, function, pPw1[0], pPw2[0], pPw3[0]);
		showResult("VerifyPassWd", result);
	}



	/*
	 * length: number of max digits in this field
	 */
	private int getEditTextDec(EditText edt, int []pBuf, int length) {

		String strText;
		int len;
		strText = edt.getText().toString();
		len = strText.length() ;
		if (len > length || len == 0)
		{
			mStrMessage = "Wrong length in text Fields ";
			Log.e(TAG, mStrMessage);
			mTextViewResult.setText(mStrMessage);
			return -1;
		}
		pBuf[0] =Integer.parseInt(edt.getText().toString());
		return 0;
	}

	/*
	 * length: number of max digits in this field
	 * */
	private int getEditTextHex(EditText edt, byte []pBuf, int length, int []pReturnLen)
	{
		byte []pByteArrary;
		String strText;
		int len;
		strText = edt.getText().toString();
		len = strText.length() ;
		Log.d(TAG, "get length="+len);
		if (len > length || len == 0 || (len%2 !=0))
		{
			mStrMessage = "Wrong length in text Fields ";
			Log.e(TAG, mStrMessage);
			mTextViewResult.setText(mStrMessage);
			return -1;
		}
		pByteArrary = MainActivity.toByteArray(strText);
		Log.d(TAG, "pByteArrary length="+pByteArrary.length);
		System.arraycopy(pByteArrary, 0, pBuf, 0, pByteArrary.length);
		pReturnLen[0] = pByteArrary.length;
		Log.d(TAG, "pBuf[0]=0x"+Integer.toHexString(pBuf[0]));
		return 0;
	}


	private boolean getRadioRWSelect()
	{
		RadioButton bt = (RadioButton) findViewById(R.id.radioWrite);
		if (bt.isChecked())
			return VERIFY_TYPE_WRITE;
		else
			return VERIFY_TYPE_READ;
	}

	private int getSpinnerSelect(Spinner s){
		long index;
		index = s.getSelectedItemId();
		return (int)index;
	}

	private int getSlotStatus(){
		int ret = SCError.READER_NO_CARD;
		byte []pCardStatus = new byte[1];

		/*detect card hotplug events*/
		ret = m88sc153.getCardStatus(pCardStatus);
		if (ret == SCError.READER_SUCCESSFUL) {
			if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_ABSENT) {
				ret = SCError.READER_NO_CARD;
			} else if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_INACTIVE) {
				ret = SCError.READER_CARD_INACTIVE;
			} else
				ret = SCError.READER_SUCCESSFUL;
		}

		return ret;
	}

	@Override
	public Reader readerSpecificInit() throws ReaderException{
		int status;
		Log.d(TAG,"readerSpecificInit");

		m88sc153 = new ReaderAT88SC153((HardwareInterface) mMyDev);
		status = m88sc153.open();
		if (status != SCError.READER_SUCCESSFUL) {
			String s = "open AT45D fail:" + SCError.errorCode2String(status);
			Log.e(TAG, s);
			throw new ReaderException(s);
		}
		return m88sc153;
	}




	@Override
	public void onCreateButtonSetup(){
		if (mUsbDev != null)
			mBtOpen.setEnabled(true);
		else
			mBtOpen.setEnabled(false);
		mBtClose.setEnabled(false);
		mBtPower.setEnabled(false);
		mBtWriteEeprom.setEnabled(false);
		mBtReadEeprom.setEnabled(false);
		mBtVerifyPassWd.setEnabled(false);
		mZoneSpinner.setEnabled(false);
		mRadioGroup.setEnabled(false);
	}

	@Override
	public void onOpenButtonSetup(){
		mBtOpen.setEnabled(false);
		mBtClose.setEnabled(true);
		mBtPower.setEnabled(true);
		mBtWriteEeprom.setEnabled(true);
		mBtReadEeprom.setEnabled(true);
		mBtVerifyPassWd.setEnabled(true);
		mZoneSpinner.setEnabled(true);
		mRadioGroup.setEnabled(true);
	}




	@Override
	public void setButtonView()
	{
		mBtOpen = (Button)findViewById(R.id.AT88SC153buttonOpen);
		mBtClose = (Button)findViewById(R.id.AT88SC153buttonClose);
		mBtPower = (Button)findViewById(R.id.AT88SC153buttonPower);
		mBtWriteEeprom = (Button)findViewById(R.id.AT88SC153WriteEep);
		mBtReadEeprom = (Button)findViewById(R.id.AT88SC153ReadEEp);
		mBtVerifyPassWd = (Button)findViewById(R.id.AT88SC153VerifyPassWd);

	}

	@Override
	public void setEditView()
	{
		mEdtAddrEEPRead = (EditText)findViewById(R.id.AT88SC153EditTextAddrEepRead);
		mEdtLenEEPRead = (EditText)findViewById(R.id.AT88SC153EditTextLenEepRead);
		mEdtAddrEEPWrite = (EditText)findViewById(R.id.AT88SC153EditTextAddrEepWrite);
		mEdtDataEEPWrite = (EditText)findViewById(R.id.AT88SC153EditTextDataEepWrite);
		mEdtVerifyPw1 = (EditText)findViewById(R.id.AT88SC153EditTextPw1);
		mEdtVerifyPw2 = (EditText)findViewById(R.id.AT88SC153EditTextPw2);
		mEdtVerifyPw3 = (EditText)findViewById(R.id.AT88SC153EditTextPw3);
	}

	
	private void setRadioButtonView()
	{
		mRadioGroup = (RadioGroup) findViewById(R.id.radioPwGroup);        
		//radioGroup.setOnCheckedChangeListener( listener);
		RadioButton bt = (RadioButton) findViewById(R.id.radioWrite);
		bt.setChecked(true);
	}

	@Override
	public void setTextView()
	{
		mTextViewResult = (TextView)findViewById(R.id.AT88SC153TextResult);
	}



	private void setupZoneSpinner(){
		String[] zoneString = {"0","1","2", "config"};
		// Initialize  spinner
		ArrayAdapter<String> mZoneAdapter = new ArrayAdapter<String>(this,
				R.layout.myspinner, zoneString);

		mZoneSpinner = (Spinner) findViewById(R.id.AT88SC153spinnerZone);
		mZoneSpinner.setAdapter(mZoneAdapter);
		mZoneSpinner.setSelection(0);
	}

	private void setupIndexSpinner(){
		String[] indexString = {"0","1"};
		// Initialize  spinner
		ArrayAdapter<String> mIndexAdapter = new ArrayAdapter<String>(this,
				R.layout.myspinner, indexString);

		mIndexSpinner = (Spinner) findViewById(R.id.AT88SC153spinnerIndex);
		mIndexSpinner.setAdapter(mIndexAdapter);
		mIndexSpinner.setSelection(0);
	}

	@Override
	public void setupViews(){
		Log.d(TAG,"setupViews");
		setRadioButtonView();
		setupZoneSpinner();
		setupIndexSpinner();
		super.setupViews();
		//setReaderSlotView();
	}

}

