package com.alcorlink.smartcard;



import amlib.ccid.Reader;
import amlib.ccid.ReaderAT88SC1608;
import amlib.ccid.ReaderException;
import amlib.ccid.SCError;
import amlib.hw.HWType;
import amlib.hw.HardwareInterface;
import amlib.hw.HardwareInterfaceUsb;
import amlib.hw.ReaderHwException;

import android.app.AlertDialog;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.app.AlertDialog.Builder;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TextView;

import java.lang.ref.WeakReference;

public class AT88SC1608_Activity extends ReaderActivity {

	private String mStrMessage;
	private ReaderAT88SC1608 m88sc1608;

	private Button mBtOpen;
	private Button mBtClose;
	private Button mBtPower;
	private Button mBtWriteUser;
	private Button mBtReadUser;
	private Button mBtWriteConfig;
	private Button mBtReadConfig;
	private Button mBtSetUserZoneAddr;
	private Button mBtVerifyPassWd;

	private RadioGroup mRadioGroup; 


	private EditText mEdtAddrUser;
	private EditText mEdtLenUser;
	private EditText mEdtDataUser;
	private EditText mEdtAddrConfig;
	private EditText mEdtLenConfig;
	private EditText mEdtDataConfig;
	private EditText mEdtAddrSet;
	private EditText mEdtVerifyPw1;
	private EditText mEdtVerifyPw2;
	private EditText mEdtVerifyPw3;


	private Spinner mZoneSpinner;


	private static final String TAG = MainActivity.TAG;
	private static final boolean VERIFY_TYPE_WRITE = false;
	private static final boolean VERIFY_TYPE_READ = true;
	private static final int MAX_WRITE_LENGTH = 256;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		setContentView(R.layout.activity_at88sc1608);
		setupViews();
		super.onCreate(savedInstanceState);
	}


	public void OnClickAT88SC1608Open (View view){
		requestDevPerm();
	}

	public void OnClickAT88SC1608Close (View view){
		new CloseTask(AT88SC1608_Activity.this).execute();
	}

	public void OnClickAT88SC1608Power(View view){
		mPowerDialog.show();
	}

	public void OnClickAT88SC1608WriteUserZone(View view){
		byte []pAddr = new byte[1];
		byte []pWrite = new byte[MAX_WRITE_LENGTH];
		int []pIntReturnLen = new int[1];
		int result;
		//get addr
		result = getEditTextHex(mEdtAddrUser, pAddr, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		//get data
		result = getEditTextHex(mEdtDataUser, pWrite, MAX_WRITE_LENGTH*2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = m88sc1608.AT88SC1608Cmd_WriteUserZone(pAddr[0], (byte)pIntReturnLen[0], pWrite);
		showResult("WriteUserZone", result);
	}

	public void OnClickAT88SC1608ReadUserZone(View view){
		byte []pAddr = new byte[1];
		int []pRxLen = new int[1];
		byte []pRead = new byte[MAX_WRITE_LENGTH];
		int []pIntReturnLen = new int[1];

		Log.d(TAG, "");
		int result;
		//get addr
		result = getEditTextHex(mEdtAddrUser, pAddr, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		//get lenght
		result = getEditTextDec(mEdtLenUser, pRxLen, 3);
		if (result != 0)
		{
			return;
		}
		result = m88sc1608.AT88SC1608Cmd_ReadUserZone(pAddr[0], (byte)pRxLen[0], pRead, pIntReturnLen);
		if (result == 0)
		{
			Log.d(TAG,"result, pIntReturnLen="+pIntReturnLen[0]);
			//shows data
			showBufResult(pIntReturnLen[0], pRead);
		}
		else
			showResult("ReadUserZone", result);
	}

	public void OnClickAT88SC1608WriteConfigZone(View view){
		byte []pAddr = new byte[1];
		byte []pWrite = new byte[MAX_WRITE_LENGTH];
		int []pIntReturnLen = new int[1];

		int result;
		//get addr
		result = getEditTextHex(mEdtAddrConfig, pAddr, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		//get data
		result = getEditTextHex(mEdtDataConfig, pWrite, MAX_WRITE_LENGTH*2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = m88sc1608.AT88SC1608Cmd_WriteConfigurationZone( pAddr[0], (byte)pIntReturnLen[0], pWrite);
		showResult("WriteConfigZone", result);
	}

	public void OnClickAT88SC1608ReadConfigZone(View view){
		byte []pAddr = new byte[1];
		int []pTxLen = new int[1];
		byte []pRead = new byte[MAX_WRITE_LENGTH];
		int []pIntReturnLen = new int[1];

		Log.d(TAG, "");
		int result;
		//get addr
		result = getEditTextHex(mEdtAddrConfig, pAddr, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		//get lenght
		result = getEditTextDec(mEdtLenConfig, pTxLen, 3);
		if (result != 0)
		{
			return;
		}
		result = m88sc1608.AT88SC1608Cmd_ReadConfigurationZone(pAddr[0], (byte)pTxLen[0], pRead, pIntReturnLen);
		if (result == 0)
		{
			Log.d(TAG,"result, pIntReturnLen="+pIntReturnLen[0]);
			//shows data
			showBufResult(pIntReturnLen[0], pRead);
		}
		else
			showResult("ReadConfigZone", result);
	}

	public void OnClickAT88SC1608SetUserZoneAddr(View view){
		byte []pAddr = new byte[1];
		int []pIntReturnLen = new int[1];
		int result;
		//get addr
		result = getEditTextHex(mEdtAddrSet, pAddr, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = m88sc1608.AT88SC1608Cmd_SetUserZoneAddress(pAddr[0]);
		showResult("SetUserZoneAddr", result);
	}

	public void OnClickAT88SC1608VerifyPassWd(View view){
		boolean function;
		byte zone;
		int []pIntReturnLen = new int[1];
		int result ;
		byte []pPw1 = new byte[1];
		byte []pPw2 = new byte[1];
		byte []pPw3 = new byte[1];

		function = getRadioRWSelect();
		zone = (byte) getSpinnerSelectZone();
		Log.d(TAG, "function="+function + "/zone="+zone);
		result = getEditTextHex(mEdtVerifyPw1, pPw1, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = getEditTextHex(mEdtVerifyPw2, pPw2, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = getEditTextHex(mEdtVerifyPw3, pPw3, 2, pIntReturnLen);
		if (result != 0)
		{
			return;
		}
		result = m88sc1608.AT88SC1608Cmd_VerifyPassword(zone, function, pPw1[0], pPw2[0], pPw3[0]);
		showResult("VerifyPassWd", result);
	}

	/*
	 * length: number of max digits in this field 
	 */
	private int getEditTextDec(EditText edt, int []pBuf, int length) {

		String strText; 
		int len;
		strText = edt.getText().toString();
		len = strText.length() ;
		if (len > length || len == 0)
		{
			mStrMessage = "Wrong length in text Fields ";
			Log.e(TAG, mStrMessage);
			mTextViewResult.setText(mStrMessage);
			return -1;
		}
		pBuf[0] =Integer.parseInt(edt.getText().toString());
		return 0;
	}

	/*
	 * length: number of max digits in this field
	 * */ 
	private int getEditTextHex(EditText edt, byte []pBuf, int length, int []pReturnLen)
	{
		byte []pByteArrary;
		String strText; 
		int len;
		strText = edt.getText().toString();
		len = strText.length() ;
		Log.d(TAG, "get length="+len);
		if (len > length || len == 0 || (len%2 !=0))
		{
			mStrMessage = "Wrong length in text Fields ";
			Log.e(TAG, mStrMessage);
			mTextViewResult.setText(mStrMessage);
			return -1;
		}
		pByteArrary = MainActivity.toByteArray(strText);
		Log.d(TAG, "pByteArrary length="+pByteArrary.length);
		System.arraycopy(pByteArrary, 0, pBuf, 0, pByteArrary.length);
		pReturnLen[0] = pByteArrary.length;
		Log.d(TAG, "pBuf[0]=0x"+Integer.toHexString(pBuf[0]));
		return 0;
	}


	private boolean getRadioRWSelect()
	{
		RadioButton bt = (RadioButton) findViewById(R.id.radioWrite);
		if (bt.isChecked())
			return VERIFY_TYPE_WRITE;
		else
			return VERIFY_TYPE_READ;
	}

	private int getSpinnerSelectZone(){
		long zone;
		zone = mZoneSpinner.getSelectedItemId();
		return (int)zone;
	}


	private int getSlotStatus(){
		int ret = SCError.READER_NO_CARD;
		byte []pCardStatus = new byte[1];

		/*detect card hotplug events*/
		ret = m88sc1608.getCardStatus(pCardStatus);
		if (ret == SCError.READER_SUCCESSFUL) {
			if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_ABSENT) {
				ret = SCError.READER_NO_CARD;
			} else if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_INACTIVE) {
				ret = SCError.READER_CARD_INACTIVE;
			} else
				ret = SCError.READER_SUCCESSFUL;
		}

		return ret;
	}

	@Override
	public Reader readerSpecificInit() throws ReaderException{
		int status;
		Log.d(TAG,"readerSpecificInit");

		m88sc1608 = new ReaderAT88SC1608((HardwareInterface) mMyDev);
		status = m88sc1608.open();
		if (status != SCError.READER_SUCCESSFUL) {
			String s = "open AT88SC1608 fail:" + SCError.errorCode2String(status);
			Log.e(TAG, s);
			throw new ReaderException(s);
		}
		return m88sc1608;
	}




	@Override
	public void onCreateButtonSetup(){
		if (mUsbDev != null)
			mBtOpen.setEnabled(true);
		else
			mBtOpen.setEnabled(false);
		mBtClose.setEnabled(false);
		mBtPower.setEnabled(false);
		mBtWriteUser.setEnabled(false);
		mBtReadUser.setEnabled(false);
		mBtWriteConfig.setEnabled(false);
		mBtReadConfig.setEnabled(false);
		mBtSetUserZoneAddr.setEnabled(false);
		mBtVerifyPassWd.setEnabled(false);
		mZoneSpinner.setEnabled(false);
		mRadioGroup.setEnabled(false);
	}

	@Override
	public  void onOpenButtonSetup(){
		mBtOpen.setEnabled(false);
		mBtClose.setEnabled(true);
		mBtPower.setEnabled(true);
		mBtWriteUser.setEnabled(true);
		mBtReadUser.setEnabled(true);
		mBtWriteConfig.setEnabled(true);
		mBtReadConfig.setEnabled(true);
		mBtSetUserZoneAddr.setEnabled(true);
		mBtVerifyPassWd.setEnabled(true);
		mZoneSpinner.setEnabled(true);
		mRadioGroup.setEnabled(true);
	}





	@Override
	public void setButtonView()
	{
		mBtOpen = (Button)findViewById(R.id.AT88SC1608buttonOpen);
		mBtClose = (Button)findViewById(R.id.AT88SC1608buttonClose);
		mBtPower = (Button)findViewById(R.id.AT88SC1608buttonPower);
		mBtWriteUser = (Button)findViewById(R.id.AT88SC1608WriteUser);
		mBtReadUser = (Button)findViewById(R.id.AT88SC1608ReadUser);
		mBtWriteConfig = (Button)findViewById(R.id.AT88SC1608WriteConfig);
		mBtReadConfig = (Button)findViewById(R.id.AT88SC1608ReadConfig);
		mBtSetUserZoneAddr = (Button)findViewById(R.id.AT88SC1608SetUserZoneAddr);
		mBtVerifyPassWd = (Button)findViewById(R.id.AT88SC1608VerifyPassWd);

	}

	@Override
	public void setEditView()
	{
		mEdtAddrUser = (EditText)findViewById(R.id.AT88SC1608EditTextAddrUser);
		mEdtLenUser = (EditText)findViewById(R.id.AT88SC1608EditTextUserLen);
		mEdtDataUser = (EditText)findViewById(R.id.AT88SC1608EditTextDataUser);
		mEdtAddrConfig = (EditText)findViewById(R.id.AT88SC1608EditTextAddrConfig);
		mEdtLenConfig = (EditText)findViewById(R.id.AT88SC1608EditTextLenConfig);
		mEdtDataConfig = (EditText)findViewById(R.id.AT88SC1608EditTextDataConfig);
		mEdtAddrSet = (EditText)findViewById(R.id.AT88SC1608EditTextAddrSet);
		mEdtVerifyPw1 = (EditText)findViewById(R.id.AT88SC1608EditTextPw1);
		mEdtVerifyPw2 = (EditText)findViewById(R.id.AT88SC1608EditTextPw2);
		mEdtVerifyPw3 = (EditText)findViewById(R.id.AT88SC1608EditTextPw3);
	}


	
	private void setRadioButtonView()
	{
		mRadioGroup = (RadioGroup) findViewById(R.id.radioPwGroup);        
		//radioGroup.setOnCheckedChangeListener( listener);
		RadioButton bt = (RadioButton) findViewById(R.id.radioWrite);
		bt.setChecked(true);
	}

	@Override
	public void setTextView()
	{
		mTextViewResult = (TextView)findViewById(R.id.AT88SC1608TextResult);
	}



	private void setupSpinner(){
		String[] lunch = {"0","1","2","3","4","5","6","7"};
		// Initialize  spinner
		ArrayAdapter<String> mZoneAdapter = new ArrayAdapter<String>(this,
				R.layout.myspinner, lunch);

		mZoneSpinner = (Spinner) findViewById(R.id.AT88SC1608spinnerZone);
		mZoneSpinner.setAdapter(mZoneAdapter);
	}


	@Override
	public void setupViews(){
		Log.d(TAG,"setupViews");
		setRadioButtonView();
		setupSpinner();
		super.setupViews();
	}


}

