package com.alcorlink.smartcard;


import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import java.lang.ref.WeakReference;

import amlib.ccid.Reader;
import amlib.ccid.ReaderException;
import amlib.ccid.SCError;
import amlib.hw.HWType;
import amlib.hw.HardwareInterface;
import amlib.hw.HardwareInterfaceUsb;
import amlib.hw.ReaderHwException;

public class ISO7816_Activity  extends ReaderActivity {
//	private Reader mReader;
	private Builder  mSlotDialog;
	private byte mSlotNum;
	//	private byte mMode;
	private Button mOpenButton;
	private Button mPowerButton;
	private Button mATRButton;
	private Button mAPDUButton;
	private Button mProtocolButton;
	private Button mSNButton;
	private Button mCloseButton;
	private EditText mEditTextApdu;
	private boolean mIsNFC;

	private String mStrMessage;


	private static final String TAG = MainActivity.TAG;


	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.d(TAG," onCreate");
		setContentView(R.layout.activity_7816);
		setupViews();

		super.onCreate(savedInstanceState);

	}


	public void OnClickISO7816Open(View view){
		Log.d(TAG, "7816OpenOnClick");
		if (mUsbDev != null)
			checkSlotNumber(mUsbDev);
	}

	public void OnClickISO7816Close(View view){
		new CloseTask(ISO7816_Activity.this).execute();
	}

	public void OnClickISO7816Power(View view){
		Log.d(TAG, "7816PowerOnClick");
		mPowerDialog.show();


	}
	public void OnClickISO7816getATR(View view){

		String atr;
		Log.d(TAG, "getATROnClick");
		atr = mReader.getAtrString();
		mTextViewResult.setText(" ATR:" + atr);

	}




	public void OnClickISO7816sendAPDUk(View view){
		byte[] pSendAPDU ;
		byte[] pRecvRes = new byte[300];
		int[] pRevAPDULen = new int[1];
		String apduStr;
		int sendLen, result;

		Log.d(TAG, "sendAPDUkOnClick");
		pRevAPDULen[0] = 300;
		apduStr = mEditTextApdu.getText().toString();
		pSendAPDU = MainActivity.toByteArray(apduStr);
		sendLen = pSendAPDU.length;

		result = mReader.transmit(pSendAPDU, sendLen, pRecvRes, pRevAPDULen);
		if (result == SCError.READER_SUCCESSFUL) {
			mTextViewResult.setText("Receive APDU: " + MainActivity.byte2String(pRecvRes, pRevAPDULen[0]));
		} else {
			mTextViewResult.setText("Fail to Send APDU: " + SCError.errorCode2String(result)
					+"(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
			Log.e(TAG, "Fail to Send APDU: " + SCError.errorCode2String(result)
					+ "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
		}

	}



	public void OnClickISO7816getProtocol(View view){
		int status;
		byte []proto = new byte[1];
		Log.d(TAG, "getProtocolOnClick");


			//status = mReader.getProtocol(proto);
			//status = mReader.badTx();
			status = mReader.getProtocol(proto);


		if (status != SCError.READER_SUCCESSFUL){
			mTextViewResult.setText("getProtocol fail: "+ SCError.errorCode2String(status)
					+ "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
			Log.e(TAG, "getProtocol fail: "+ SCError.errorCode2String(status)
					+ "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
		}else{
			if (proto[0] == Reader.CCID_PROTOCOL_T0)
				mTextViewResult.setText("T0");
			if (proto[0] == Reader.CCID_PROTOCOL_T1)
				mTextViewResult.setText("T1");
		}
	}

	public void OnClickISO7816getSN(View view){
		//boolean status;
//		int status = 0;
//		byte[] pLen = new byte[1];
//		byte []pSN;
//		Log.d(TAG, "getSNOnClick");
//		byte DEFAULT_SN_LEN = 32;
//		pLen[0] = DEFAULT_SN_LEN;
//		pSN = new byte[pLen[0]];
//
//		status = mReader.getSN(pSN, pLen);
//
//		if (status != SCError.READER_SUCCESSFUL){
//
//			mTextViewResult.setText("getSN fail: " +SCError.errorCode2String(status)
//					+ "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
//		}else{
//			String str = new String(pSN);
//			//mTextSN.setText(logBuffer(pSN, pLen[0]));
//			mTextViewResult.setText(str);
//		}
	}


	private void checkSlotNumber(UsbDevice uDev){
		mIsNFC = false;
		if(uDev.getProductId() == 0x9522 || uDev.getProductId() == 0x9525 )
			setReaderTwoSlotView();
		else if(uDev.getProductId() == 0x9526 ) 
			setReaderNFCSlotView();
		else if(uDev.getProductId() == 0x9572 )
			setReaderNFCSlotView();
		
		else{
			mSlotNum = (byte)0;
			requestDevPerm();
		}
	}





	private int getSlotStatus(){
		int ret = SCError.READER_NO_CARD;
		byte []pCardStatus = new byte[1];

		/*detect card hotplug events*/

		ret = mReader.getCardStatus(pCardStatus);
		if (ret == SCError.READER_SUCCESSFUL) {
			if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_ABSENT) {
				ret = SCError.READER_NO_CARD;
			} else if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_INACTIVE) {
				ret = SCError.READER_CARD_INACTIVE;
			} else {
				ret = SCError.READER_SUCCESSFUL;
			}
		}

		return ret;
	}



	@Override
	public Reader readerSpecificInit() throws ReaderException {
		int status;
		Log.d(TAG,"readerSpecificInit");
		Reader reader = new Reader((HardwareInterface) mMyDev);
		status = reader.open();
		if (status != SCError.READER_SUCCESSFUL) {
			String s = "open Reader fail:" + SCError.errorCode2String(status);
			Log.e(TAG, s);
			throw new ReaderException(s);
		}
		reader.setSlot(mSlotNum);
		return reader;

	}


	@Override
	public void onCreateButtonSetup(){
		if (mUsbDev != null)
			mOpenButton.setEnabled(true);
		else
			mOpenButton.setEnabled(false);
		mPowerButton.setEnabled(false);
		mATRButton.setEnabled(false);
		mAPDUButton.setEnabled(false);
		mSNButton.setEnabled(false);
		mProtocolButton.setEnabled(false);
		mCloseButton.setEnabled(false);
	}


	@Override
	public void onOpenButtonSetup(){
		mOpenButton.setEnabled(false);
		mPowerButton.setEnabled(true);
		mATRButton.setEnabled(true);
		mAPDUButton.setEnabled(true);
		mSNButton.setEnabled(true);
		if (!mIsNFC)//meaningless for NFC
			mProtocolButton.setEnabled(true);
		mCloseButton.setEnabled(true);
	}


	private void debugShowCurrentVoltage() {
		Reader.VoltageSwitch []vol = new Reader.VoltageSwitch[1];
		mReader.getCurrentVoltage(vol);
		Log.d(TAG,"Current Voltage is "+ vol[0].toString());
	}

    private int poweronVoltageQuery(){
        int result = SCError.READER_SUCCESSFUL;
        Log.d(TAG,"poweronVoltageQuery");

        result = mReader.setPower(Reader.CCID_POWERON, Reader.VoltageSwitch.Voltage1_8);
		debugShowCurrentVoltage();
        if (result == SCError.READER_SUCCESSFUL) {
			return result;
		}
		result = mReader.setPower(Reader.CCID_POWERON, Reader.VoltageSwitch.Voltage3);
		debugShowCurrentVoltage();
		if (result == SCError.READER_SUCCESSFUL) {
			return result;
		}
		result = mReader.setPower(Reader.CCID_POWERON, Reader.VoltageSwitch.Auto);
		debugShowCurrentVoltage();
        Log.d(TAG,"poweronVoltageQuery exit");

        return result;
    }



	private void setReaderNFCSlotView(){
		String [] s = new String[] {"SmartCard","NFC"};
		setReaderSlotView(s);
	}
	private void setReaderTwoSlotView(){
		String [] s = new String[] {"slot-0","slot-1"};
		setReaderSlotView(s);
	}

	private void setReaderSlotView(String []arraySlot){

		mSlotDialog = new AlertDialog.Builder(this);
		DialogInterface.OnClickListener Select = new DialogInterface.OnClickListener(){

			@Override  
			public void onClick(DialogInterface dialog, int which) {
				mSlotNum = (byte) which;
				if (which == 0x01)// it is NFC slot
					mIsNFC = true;
			}  
		};

		DialogInterface.OnClickListener OkClick = new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int which) {
				requestDevPerm();
			}
		}; 

		mSlotDialog.setPositiveButton("OK",OkClick );
		mSlotDialog.setTitle("Select Slot Number");
		mSlotDialog.setSingleChoiceItems(arraySlot, 0, Select) ;
		mSlotDialog.show();
	}



	@Override
	public void setupViews(){
		setReaderSlotView();
		super.setupViews();
	}

	@Override
	public void setButtonView() {
		mPowerButton = (Button)findViewById(R.id.ISO7816buttonPower);
		mOpenButton = (Button)findViewById(R.id.ISO7816buttonOpen);
		mATRButton  = (Button)findViewById(R.id.ISO7816buttonATR);
		mAPDUButton = (Button)findViewById(R.id.ISO7816buttonAPDU);
		mProtocolButton = (Button)findViewById(R.id.ISO7816buttonProtocol);
		mSNButton = (Button)findViewById(R.id.ISO7816buttonSN);
		mCloseButton = (Button)findViewById(R.id.ISO7816buttonClose);
	}

	@Override
	public  void setEditView()
	{

		mEditTextApdu = (EditText)findViewById(R.id.ISO7816editTextAPDU);
		mEditTextApdu.setText("A0A40000023F00");
	}

	@Override
	public  void setTextView()
	{
		mTextViewResult = (TextView)findViewById(R.id.ISO7816textResult);
	}
	private void setReaderSlotView(){
		final String [] arraySlot = new String[] {"Slot:0","Slot:1"};
		mSlotDialog = new AlertDialog.Builder(this);
		DialogInterface.OnClickListener Select = new DialogInterface.OnClickListener(){

			@Override  
			public void onClick(DialogInterface dialog, int which) {
				mSlotNum = (byte) which;
			}  
		};

		DialogInterface.OnClickListener OkClick = new DialogInterface.OnClickListener()
		{
			public void onClick(DialogInterface dialog, int which) {
				requestDevPerm();
			}
		}; 
		mSlotDialog.setPositiveButton("OK",OkClick );
		mSlotDialog.setTitle("Select Slot Number");
		mSlotDialog.setSingleChoiceItems(arraySlot, 0, Select) ;
	}

}
