package com.alcorlink.smartcard;


import android.app.AlertDialog.Builder;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import amlib.ccid.Reader;
import amlib.ccid.ReaderException;
import amlib.ccid.SCError;
import amlib.hw.HWType;
import amlib.hw.HardwareInterface;
import amlib.hw.HardwareInterfaceUsb;
import amlib.hw.ReaderHwException;

public class NFC_Activity extends ReaderActivity {

    private String mStrMessage;

    private Button mBtOpen;
    private Button mBtClose;
    private Button mBtConnect;
    private Button mBt14443Send;
    private Button mBtSend;
    private EditText mEdt4443EdtData;
    private EditText mEdtGenericData;


    private Builder mWarnDialog;

    private static final String TAG = MainActivity.TAG;


    protected static final int EVENT_SET_RESULT = 221;
    protected static final int EVENT_CCID_RESULT = 222;
    protected static final int EVENT_ERROR_DIALOG = 223;

    private static final int EVENT_UPDATE_TRAILER = 225;

    private final String CMDgetID = "FFCA0000";
    private final String CMDselectPPSE = "00A404000E325041592E5359532E4444463031";
    private final String CMDreadRandomValue = "0084000008";
    private final String CMDread6bytesAtAddressZero = "00B000000f";
    private byte mSlot;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        setContentView(R.layout.activity_nfc);
        setupViews();
        setupHandler();
        super.onCreate(savedInstanceState);
    }


    public void OnClickNFCOpen(View view) {
        requestDevPerm();
    }

    public void OnClick1NFCClose(View view) {
        new CloseTask(this).execute();
    }

    public void OnClickNFCConnect(View view) {
        int ret;
        ret = poweron();
        if (ret == SCError.READER_SUCCESSFUL) {
            mTextViewResult.setText("power on successfully");
            String atr;

            atr = mReader.getAtrString();
            mTextViewResult.append("\nATR:" + atr);
        } else if (SCError.maskStatus(ret) == SCError.READER_NO_CARD) {
            mTextViewResult.setText("Card Absent");
        }
    }


    public void OnClick14443SendCommand(View view) {
        byte[] pRecvRes = new byte[300];
        int[] pRevAPDULen = new int[1];
        String cmd = mEdt4443EdtData.getText().toString();
        byte[] apdu = MainActivity.toByteArray(cmd);

        for (int i = 0; i < apdu.length; i++) {
            Log.d("XXXXXXXXXXX", Integer.toHexString(apdu[i]));
        }
        readerSendApdu(apdu);
    }


    public void OnClickNFCSendAPDU(View view) {
        byte[] pRecvRes = new byte[300];
        int[] pRevAPDULen = new int[1];
        String cmd = mEdt4443EdtData.getText().toString();
        byte[] apdu = MainActivity.toByteArray(cmd);
        String s = "apdu length=" + apdu.length;
        Log.d(TAG, s);
        if (apdu.length == 0) {
            mTextViewResult.setText(s);
            return;
        }
        for (int i = 0; i < apdu.length; i++) {
            Log.d(TAG, Integer.toHexString(apdu[i]));
        }
        readerSendApdu(apdu);
    }


    /*
     * length: number of max digits in this field
     */
    private int getEditTextDec(EditText edt, int[] pBuf, int length) {

        String strText;
        int len;
        if (edt.length() == 0) {
            //showErrorDialog("The block decimal value is empty");
            return -1;
        }
        strText = edt.getText().toString();
        len = strText.length();
        if (len > length || len == 0) {
            showResult("Wrong length in text field");
            showErrorDialog("Wrong length in text field");
            return -1;
        }
        pBuf[0] = Integer.parseInt(edt.getText().toString());
        return 0;
    }

    /*
     * length: number of max digits in this field
     * */

    private int getEditTextHex(EditText edt, byte[] pBuf, int length, int[] pReturnLen) {
        byte[] pByteArrary;
        String strText;
        int len;
        if (edt.length() == 0) {
            //showErrorDialog("The edit field is empty");
            pReturnLen[0] = 0;
            return -1;
        }
        strText = edt.getText().toString();
        len = strText.length();

        if (len > length || len == 0 || (len % 2 != 0)) {
//			showResult("Wrong length in Key field");
//			showErrorDialog("Wrong length in Key field");
            return -1;
        }
        pByteArrary = MainActivity.toByteArray(strText);
        System.arraycopy(pByteArrary, 0, pBuf, 0, pByteArrary.length);
        pReturnLen[0] = pByteArrary.length;

        return 0;
    }





    @Override
    public Reader readerSpecificInit() throws ReaderException {
        int status;
        boolean init;//
        boolean isTwoInterface = DeviceCheck.isTwoInterface(mUsbDev.getProductId());
        Reader reader;
        Log.d(TAG, "readerSpecificInit");

        try {
            UsbManager manager = (UsbManager) getSystemService(Context.USB_SERVICE);
            /*If there are two interfaces, NFC takes the second interface*/
            if (isTwoInterface == true) //9573 has two interfaces
                init = mMyDev.initSecondInterface(manager, mUsbDev);
            else
                init = mMyDev.init(manager, mUsbDev);
            if (!init) {
                String s = "Device init fail";
                Log.e(TAG, s);
                throw new ReaderException(s);
            }
        } catch (ReaderHwException e) {
            throw new ReaderException(e);
        }

        reader = new Reader((HardwareInterface) mMyDev);
        if (isTwoInterface == true)
            mSlot = 0;
        else
            mSlot = 1;
        Log.d(TAG, "Open with slot:" + mSlot);
        status = reader.open(mSlot);
        if (status != SCError.READER_SUCCESSFUL) {
            String s = "open NFC Reader fail:" + SCError.errorCode2String(status);
            Log.e(TAG, s);
            throw new ReaderException(s);
        }
        return reader;


    }


    private Handler mMessageHandler;


    @Override
    public void onCreateButtonSetup() {
        if (mUsbDev != null)
            mBtOpen.setEnabled(true);
        else
            mBtOpen.setEnabled(false);
        mBtClose.setEnabled(false);
        mBtConnect.setEnabled(false);
        mBt14443Send.setEnabled(false);
        mBtSend.setEnabled(false);

    }


    @Override
    public void onOpenButtonSetup() {
        mBtOpen.setEnabled(false);
        mBtClose.setEnabled(true);
        mBtConnect.setEnabled(true);
        mBt14443Send.setEnabled(true);
        mBtSend.setEnabled(true);
    }


    private void readerSendApdu(byte[] apdu) {
        byte[] pRecvRes = new byte[300];
        int[] pRevAPDULen = new int[1];
        int result = mReader.transmit(apdu, apdu.length, pRecvRes, pRevAPDULen);
        if (result != SCError.READER_SUCCESSFUL) {
            mTextViewResult.setText("Fail to Send APDU: " + SCError.errorCode2String(result)
                    + "(0x" + Integer.toHexString(mReader.getCmdFailCode()) + ")");
            Log.e(TAG, "Fail to Send APDU: " + SCError.errorCode2String(result)
                    + "(0x" + Integer.toHexString(mReader.getCmdFailCode()) + ")");
        }
        mTextViewResult.setText("Receive APDU: " + MainActivity.byte2String(pRecvRes, pRevAPDULen[0]));
    }


    @Override
    public void setButtonView() {
        mBtOpen = (Button) findViewById(R.id.iso14443buttonOpen);
        mBtClose = (Button) findViewById(R.id.iso14443buttonClose);
        mBtConnect = (Button) findViewById(R.id.iso14443buttonPower);
        mBt14443Send = (Button) findViewById(R.id.BtNfc14443);
        mBtSend = (Button) findViewById(R.id.BtNfcGenericSend);
    }

    @Override
    public void setEditView() {
        mEdt4443EdtData = findViewById(R.id.ISO14443EdtData);
//        mEdt4443EdtData.setText("");
        mEdtGenericData = findViewById(R.id.NFCEdtDataGeneric);
//        mEdtGenericData.setText("");
    }


    private void setRadioView() {
        RadioGroup rg14443 = findViewById(R.id.RadioGroup14443Cmd);
        rg14443.setOnCheckedChangeListener(radioChangeListener);
    }


    private RadioGroup.OnCheckedChangeListener radioChangeListener = new RadioGroup.OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(RadioGroup group, int checkedId) {
            Log.d(TAG, "onCheckedChanged ");
            switch (checkedId) {
                case R.id.radio14443Id:
                    Log.d(TAG, "");
                    mEdt4443EdtData.setText(CMDgetID);

                    break;
                case R.id.radio14443PPSE:
                    Log.d(TAG, "");
                    mEdt4443EdtData.setText(CMDselectPPSE);
                    break;
                case R.id.radio14443ReadRand:
                    Log.d(TAG, "");
                    mEdt4443EdtData.setText(CMDreadRandomValue);

                    break;
                case R.id.radio14443Read4:
                    Log.d(TAG, "");
                    mEdt4443EdtData.setText(CMDread6bytesAtAddressZero);
                    break;

            }
        }
    };

    @Override
    public void setTextView() {

        mTextViewResult = (TextView) findViewById(R.id.iso14443TextResult);
    }

    private void setupHandler() {
        mMessageHandler = new Handler(Looper.getMainLooper()) {
            @Override
            public void handleMessage(Message message) {
                String msg;
                byte[] pData;
                switch (message.what) {
                    case EVENT_SET_RESULT:
                        msg = message.getData().getString("RESULT");
                        mTextViewResult.setText(msg);
                        break;
                    case EVENT_ERROR_DIALOG:
                        msg = message.getData().getString("ERROR");
                        mWarnDialog.setMessage(msg);
                        mWarnDialog.show();
                        break;


                    case EVENT_CCID_RESULT:
                        int result = message.getData().getInt("CCID_RESULT");
                        if (result == SCError.READER_SUCCESSFUL) {
                            mTextViewResult.setText("Success");
                        } else {
                            mTextViewResult.setText("Fail:" + SCError.errorCode2String(result) + "(" + Integer.toHexString(result) + ")");
                        }
                }
            }
        };
    }


    @Override
    public void setupViews() {
        Log.d(TAG, "setupViews");
        setRadioView();
        setUpWarnDialog();
        super.setupViews();
    }


    private void setUpWarnDialog() {

        mWarnDialog = new Builder(this);

        DialogInterface.OnClickListener errorClick = new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {

            }
        };
        mWarnDialog.setTitle("Error!!");
        mWarnDialog.setPositiveButton("OK", errorClick);
        mWarnDialog.setCancelable(true);
    }


    private void showErrorDialog(String s) {
        Bundle countBundle = new Bundle();
        Message msg = new Message();
        msg.what = EVENT_ERROR_DIALOG;
        countBundle.putString("ERROR", s);
        msg.setData(countBundle);
        mMessageHandler.sendMessage(msg);
    }

    private void showResult(String s) {

        Bundle countBundle = new Bundle();
        Message msg = new Message();
        msg.what = EVENT_SET_RESULT;
        countBundle.putString("RESULT", s);
        msg.setData(countBundle);
        //Toast.makeText(this.getApplicationContext(), e.toString(),Toast.LENGTH_SHORT).show();
        mMessageHandler.sendMessage(msg);
    }

    private void showResult(int result) {
        Bundle countBundle = new Bundle();
        Message msg = new Message();
        msg.what = EVENT_CCID_RESULT;
        countBundle.putInt("CCID_RESULT", result);
        msg.setData(countBundle);
        //Toast.makeText(this.getApplicationContext(), e.toString(),Toast.LENGTH_SHORT).show();
        mMessageHandler.sendMessage(msg);

    }


    private void value2BlockHex(int value, byte adr, byte[] pHex) {

        pHex[0] = (byte) (0x000000ff & value);
        pHex[1] = (byte) ((0x0000ff00 & value) >> 8);
        pHex[2] = (byte) ((0x00ff0000 & value) >> 16);
        pHex[3] = (byte) ((0xff000000 & value) >> 24);
        pHex[4 + 0] = (byte) (0x000000ff & ~value);
        pHex[4 + 1] = (byte) ((0x0000ff00 & ~value) >> 8);
        pHex[4 + 2] = (byte) ((0x00ff0000 & ~value) >> 16);
        pHex[4 + 3] = (byte) ((0xff000000 & ~value) >> 24);
        pHex[8 + 0] = (byte) (0x000000ff & value);
        pHex[8 + 1] = (byte) ((0x0000ff00 & value) >> 8);
        pHex[8 + 2] = (byte) ((0x00ff0000 & value) >> 16);
        pHex[8 + 3] = (byte) ((0xff000000 & value) >> 24);

        pHex[12 + 0] = (byte) adr;
        pHex[12 + 1] = (byte) ~adr;
        pHex[12 + 2] = (byte) adr;
        pHex[12 + 3] = (byte) ~adr;

    }
}
