package com.alcorlink.smartcard;

import android.app.AlertDialog;
import android.app.Application;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.hardware.usb.UsbDevice;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import com.alcorlink.serialport.HardwareInterfaceSerial;
import android.serialport.SerialPortFinder;

import java.io.IOException;
import java.lang.ref.WeakReference;

import amlib.ccid.Reader;
import amlib.ccid.ReaderException;
import amlib.ccid.SCError;
import amlib.hw.HWType;

import amlib.hw.ReaderHwException;

public class RS232_ISO7816_Activity  extends AppCompatActivity {
    private Reader mReader;
    private HardwareInterfaceSerial mMyDev;
    private AlertDialog.Builder mPowerDialog;
    //	private byte mMode;
    private Button mOpenButton;
    private Button mPowerButton;
    private Button mATRButton;
    private Button mAPDUButton;
    private Button mProtocolButton;
    private Button mCloseButton;
    private TextView mTextViewResult;
    private EditText mEditTextApdu;

    private ProgressDialog mWaitProgress;

    private String mStrMessage;


    private static final String TAG = MainActivity.TAG;
    private static final String ACTION_USB_PERMISSION = "com.alcorlink.smartcard.USB_PERMISSION";


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_rs232);
        setupViews();

    }

    @Override
    protected void   onDestroy() {
        Log.e(TAG,"onDestroy");
        if (mMyDev != null)
        {
            try {
                mMyDev.Close();
            } catch (ReaderHwException e) {
                e.printStackTrace();
            }
        }
        super.onDestroy();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if ((keyCode == KeyEvent.KEYCODE_BACK)) {
            try {
                closeReader(RS232_ISO7816_Activity.this);

            } catch (Exception e) {
                mStrMessage = "Get Exception : " + e.getMessage();
                //mTextViewResult.setText( mStrMessage);
            }
            Log.d(TAG,"finish");
            RS232_ISO7816_Activity.this.finish();
        }
        return super.onKeyDown(keyCode, event);

    }




    private int initReader()
    {

        mMyDev = new HardwareInterfaceSerial(HWType.eRS232);
        int Status = 0;
        Log.d(TAG, "InitReader");
        try {
            mMyDev.Init(selectedTty,38400);
        } catch (ReaderHwException e) {
            mStrMessage = "Get Exception : " + e.getMessage();
            return -1;
        }

        try {
            mReader = new Reader(mMyDev);
        }
        catch (ReaderException e)
        {
            mStrMessage = "Get Exception : " + e.getMessage();
            Log.e(TAG, "InitReader fail "+ mStrMessage);
            return -1;
        }

        mReader.setSlot((byte) 0);
        Status  = mReader.open();
        if (Status != 0) {
            mStrMessage = "open Reader fail : " + SCError.errorCode2String(Status);
            Log.e(TAG, "open Reader fail : " + SCError.errorCode2String(Status));
        }
        return Status;
    }


    public void OnClickISO7816Open(View view){
        Log.d(TAG, "7816OpenOnClick");
        if (selectedTty == null){
            mTextViewResult.setText("No TTY been found");
            return;
        }

        int ret = initReader();
        if (ret == SCError.READER_SUCCESSFUL)
            mTextViewResult.setText("Open successfully");
        else {
            mTextViewResult.setText("Open fail:" +  mStrMessage  );
        }
    }

    public void OnClickISO7816Close(View view){
        new RS232_ISO7816_Activity.CloseTask(RS232_ISO7816_Activity.this).execute();
    }

    public void OnClickISO7816Power(View view){
        Log.d(TAG, "7816PowerOnClick");
        if (mReader == null) {
            mTextViewResult.setText("No device or device initial fail"  );
            return;
        }
        mPowerDialog.show();


    }
    public void OnClickISO7816getATR(View view){

        String atr;
        Log.d(TAG, "getATROnClick");
        if (mReader == null) {
            mTextViewResult.setText("No device or device initial fail"  );
            return;
        }
        atr = mReader.getAtrString();
        mTextViewResult.setText(" ATR:" + atr);

    }




    public void OnClickISO7816sendAPDUk(View view){
        byte[] pSendAPDU ;
        byte[] pRecvRes = new byte[300];
        int[] pRevAPDULen = new int[1];
        String apduStr;
        int sendLen, result;

        Log.d(TAG, "sendAPDUkOnClick");
        if (mReader == null) {
            mTextViewResult.setText("No device or device initial fail"  );
            return;
        }
        pRevAPDULen[0] = 300;
        apduStr = mEditTextApdu.getText().toString();
        pSendAPDU = MainActivity.toByteArray(apduStr);
        sendLen = pSendAPDU.length;

        result = mReader.transmit(pSendAPDU, sendLen, pRecvRes, pRevAPDULen);
        if (result == SCError.READER_SUCCESSFUL) {
            mTextViewResult.setText("Receive APDU: " + MainActivity.byte2String(pRecvRes, pRevAPDULen[0]));
        } else {
            mTextViewResult.setText("Fail to Send APDU: " + SCError.errorCode2String(result)
                    +"(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
            Log.e(TAG, "Fail to Send APDU: " + SCError.errorCode2String(result)
                    + "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
        }

    }



    public void OnClickISO7816getProtocol(View view){
        int status;
        byte []proto = new byte[1];
        Log.d(TAG, "getProtocolOnClick");
        if (mReader == null) {
            mTextViewResult.setText("No device or device initial fail"  );
            return;
        }
        status = mReader.getProtocol(proto);


        if (status != SCError.READER_SUCCESSFUL){
            mTextViewResult.setText("getProtocol fail: "+ SCError.errorCode2String(status)
                    + "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
            Log.e(TAG, "getProtocol fail: "+ SCError.errorCode2String(status)
                    + "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
        }else{
            if (proto[0] == Reader.CCID_PROTOCOL_T0)
                mTextViewResult.setText("T0");
            if (proto[0] == Reader.CCID_PROTOCOL_T1)
                mTextViewResult.setText("T1");
        }
    }

    public void OnClickISO7816getSN(View view){
        //boolean status;

        mTextViewResult.setText("Not support");

    }






    private static int closeReader(RS232_ISO7816_Activity act){
        Log.d(TAG, "Closing reader...");
        int ret = 0;

        if (act.mReader != null)
        {
            ret = act.mReader.close();

        }
        try {
            act.mMyDev.Close();
            act.mReader = null;
        } catch (ReaderHwException e) {
            e.printStackTrace();
        }
        return ret;
    }



    private static class CloseTask extends AsyncTask<Void, Void, Integer> {
        private final WeakReference<RS232_ISO7816_Activity> weakActivity;

        CloseTask(RS232_ISO7816_Activity myActivity) {
            this.weakActivity = new WeakReference<RS232_ISO7816_Activity>(myActivity);
        }

        @Override
        protected void onPreExecute() {
            RS232_ISO7816_Activity activity = weakActivity.get();
            setUpWaitDialog(activity, "Closing Reader");
        }
        @Override
        protected Integer doInBackground(Void... params) {
            int status = 0;
            RS232_ISO7816_Activity activity = weakActivity.get();
            try {
                do{
                    status = closeReader(activity);
                }while(SCError.maskStatus(status) == SCError.READER_CMD_BUSY);

            } catch (Exception e) {
                activity.mStrMessage = "Get Exception : " + e.getMessage();
                //mTextViewResult.setText( mStrMessage);
                return -1;
            }
            return status;
        }

        @Override
        protected void onPostExecute(Integer result) {
            RS232_ISO7816_Activity activity = weakActivity.get();
            if (result != 0) {
                activity.mTextViewResult.setText("Close fail: "+ SCError.errorCode2String(result) +". " + activity.mStrMessage);
                Log.e(TAG, "Close fail: "+ SCError.errorCode2String(result));
            }else{
                activity.mTextViewResult.setText("Close successfully");
                Log.e(TAG,"Close successfully");
            }

            activity.mWaitProgress.dismiss();
        }
    }


    private int getSlotStatus(){
        int ret = SCError.READER_NO_CARD;
        byte []pCardStatus = new byte[1];

        ret = mReader.getCardStatus(pCardStatus);
        if (ret == SCError.READER_SUCCESSFUL) {
            if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_ABSENT) {
                ret = SCError.READER_NO_CARD;
            } else if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_INACTIVE) {
                ret = SCError.READER_CARD_INACTIVE;
            } else {
                ret = SCError.READER_SUCCESSFUL;
            }
        }

        return ret;
    }





    private void debugShowCurrentVoltage() {
        Reader.VoltageSwitch []vol = new Reader.VoltageSwitch[1];
        mReader.getCurrentVoltage(vol);
        Log.d(TAG,"Current Voltage is "+ vol[0].toString());
    }

    private int poweronVoltageQuery(){
        int result = SCError.READER_SUCCESSFUL;
        Log.d(TAG,"poweronVoltageQuery");

        result = mReader.setPower(Reader.CCID_POWERON, Reader.VoltageSwitch.Voltage1_8);
        debugShowCurrentVoltage();
        if (result == SCError.READER_SUCCESSFUL) {
            return result;
        }
        result = mReader.setPower(Reader.CCID_POWERON, Reader.VoltageSwitch.Voltage3);
        debugShowCurrentVoltage();
        if (result == SCError.READER_SUCCESSFUL) {
            return result;
        }
        result = mReader.setPower(Reader.CCID_POWERON, Reader.VoltageSwitch.Auto);
        debugShowCurrentVoltage();
        Log.d(TAG,"poweronVoltageQuery exit");

        return result;
    }


    private int poweron(){
        int result = SCError.READER_SUCCESSFUL;
        Log.d(TAG,"poweron");
        //check slot status first
        result = getSlotStatus();
        switch (result)
        {
            case SCError.READER_NO_CARD:
                mTextViewResult.setText("Card Absent");
                Log.d(TAG,"Card Absent");
                return SCError.READER_NO_CARD;
            case SCError.READER_CARD_INACTIVE:
            case SCError.READER_SUCCESSFUL:
                break;
            default://returns other error case
                return result;
        }

        result = poweronVoltageQuery();
        if (result != SCError.READER_SUCCESSFUL)
            result = mReader.setPower(Reader.CCID_POWERON);
        Log.d(TAG,"power on exit");

        return result;
    }


    private int poweroff(){
        int result =  SCError.READER_SUCCESSFUL;
        Log.d(TAG,"poweroff");
        result = getSlotStatus();
        switch (result)
        {
            case SCError.READER_NO_CARD:
                mTextViewResult.setText("Card Absent");
                Log.d(TAG,"Card Absent");
                return SCError.READER_NO_CARD;
            case SCError.READER_CARD_INACTIVE:
            case SCError.READER_SUCCESSFUL:
                break;
            default://returns other error case
                return result;
        }
        //----------poweroff card------------------

        result = mReader.setPower(Reader.CCID_POWEROFF);

        return result;
    }

    ArrayAdapter mTtyAdapter;
    Spinner mReaderSpinner;
    String selectedTty = null;


    private String[] searchTty() throws IOException {
        String[] ttyList = null;

        SerialPortFinder serialPortFinder;
        serialPortFinder = new SerialPortFinder();

        ttyList = serialPortFinder.getAllDevicesPath();

        return ttyList;
    }

    private void setupReaderSpinner(){

        // Initialize reader spinner
        mTtyAdapter = new ArrayAdapter<String>(this,
                R.layout.myspinner);

        String[] ttyList = new String[0];
        try {
            ttyList = searchTty();
        } catch (NullPointerException e){
            Log.e(TAG, "NullPointerException" +e.toString());
            e.printStackTrace();
            mTextViewResult.setText("No tty been found");
            return;

        } catch (IOException e2) {
            Log.e(TAG, "IOException" +e2.toString());
            e2.printStackTrace();
            mTextViewResult.setText("No tty been found");
            return;
        }

        mTtyAdapter.addAll(ttyList);

        mReaderSpinner = (Spinner) findViewById(R.id.spinnerTty);
        if (mReaderSpinner == null)
            Log.d(TAG, "mReaderSpinner is null");
        mReaderSpinner.setAdapter(mTtyAdapter);

        mReaderSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
            @Override
            public void onItemSelected(AdapterView<?> arg0, View arg1,int position, long arg3) {
                selectedTty = (String) mReaderSpinner.getSelectedItem();
            }
            @Override
            public void onNothingSelected(AdapterView<?> arg0) {
                // TODO Auto-generated method stub
            }
        });
    }


    private static void setUpWaitDialog(RS232_ISO7816_Activity act, String msg){
        act.mWaitProgress.setMessage(msg);
        act.mWaitProgress.setCancelable(false);
        act.mWaitProgress.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        act.mWaitProgress.show();
    }

    private void setupViews(){
        mPowerButton = (Button)findViewById(R.id.ISO7816buttonPower);
        mOpenButton = (Button)findViewById(R.id.ISO7816buttonOpen);
        mATRButton  = (Button)findViewById(R.id.ISO7816buttonATR);
        mAPDUButton = (Button)findViewById(R.id.ISO7816buttonAPDU);
        mProtocolButton = (Button)findViewById(R.id.ISO7816buttonProtocol);
        mCloseButton = (Button)findViewById(R.id.ISO7816buttonClose);


        mTextViewResult = (TextView)findViewById(R.id.ISO7816textResult);
        mEditTextApdu = (EditText)findViewById(R.id.ISO7816editTextAPDU);
        mEditTextApdu.setText("A0A40000023F00");

        mWaitProgress =  new ProgressDialog(RS232_ISO7816_Activity.this);

        setPowerView();
        setupReaderSpinner();
    }


    private void setPowerView(){
        mPowerDialog = new AlertDialog.Builder(this);
        DialogInterface.OnClickListener poweroffClick = new DialogInterface.OnClickListener(){
            @Override
            public void onClick(DialogInterface dialog, int which) {
                TextView textViewResult;
                int ret;
                textViewResult = (TextView)findViewById(R.id.ISO7816textResult);
                ret =	poweroff();
                if (ret == SCError.READER_SUCCESSFUL)
                    textViewResult.setText("power off successfully");
                else if (SCError.maskStatus(ret) == SCError.READER_NO_CARD){
                    textViewResult.setText("Card Absent");
                }
                else
                {
                    textViewResult.setText("power off fail:"+  SCError.errorCode2String(ret)
                            +"(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
                }
            }
        };

        DialogInterface.OnClickListener poweronClick = new DialogInterface.OnClickListener()
        {
            public void onClick(DialogInterface dialog, int which) {
                TextView textViewResult;
                int ret;
                textViewResult = (TextView)findViewById(R.id.ISO7816textResult);
                ret = poweron();
                if (ret == SCError.READER_SUCCESSFUL)
                {
                    textViewResult.setText("power on successfully");
                }
                else if (SCError.maskStatus(ret) == SCError.READER_NO_CARD){
                    textViewResult.setText("Card Absent");
                }
                else
                {
                    textViewResult.setText("power on fail:"+  SCError.errorCode2String(ret) + "(0x"+Integer.toHexString(mReader.getCmdFailCode())+")");
                }
            }
        };
        mPowerDialog.setTitle("Set Power");
        mPowerDialog.setNeutralButton("PowerON", poweronClick);
        mPowerDialog.setPositiveButton("PowerOFF", poweroffClick);

    }


}
