package com.alcorlink.smartcard;

import amlib.ccid.Reader;
import amlib.ccid.ReaderAT45D041;
import amlib.ccid.ReaderException;
import amlib.ccid.SCError;
import amlib.hw.HWType;
import amlib.hw.HardwareInterface;
import amlib.hw.HardwareInterfaceUsb;
import amlib.hw.ReaderHwException;

import android.app.AlertDialog;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.app.AlertDialog.Builder;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import java.lang.ref.WeakReference;

public abstract class ReaderActivity extends AppCompatActivity {
    private String mStrMessage;
    public HardwareInterfaceUsb mMyDev;
    public Reader mReader;
    public UsbDevice mUsbDev;
    private UsbManager mManager;
    private PendingIntent mPermissionIntent;
    public TextView mTextViewResult;
    public ProgressDialog mWaitProgress;
    public Builder  mPowerDialog;
    private static final String TAG = MainActivity.TAG;
    private static final String ACTION_USB_PERMISSION = "com.alcorlink.smartcard.USB_PERMISSION";
    protected static final byte CARD_TYPE_SINGLE = 0;
    protected static final byte CARD_TYPE_DOUBLE = 1;

    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        Intent intent = this.getIntent();
        Bundle b = intent.getExtras();
        if (b != null)
        {
            mUsbDev = (UsbDevice) b.getParcelable(USB_SERVICE);
        }
        if (mUsbDev != null)
        {
            Log.d(TAG, "mdev=" + Integer.toHexString(mUsbDev.getProductId()));
        }
        setupViews();

        try {
            mMyDev = new HardwareInterfaceUsb(HWType.eUSB);
        }catch(Exception e){
            mStrMessage = "Get Exception : " + e.getMessage();
            Log.e(TAG, mStrMessage);
            return;
        }
        // Get USB manager
        mManager = (UsbManager) getSystemService(Context.USB_SERVICE);
        toRegisterReceiver();
        if (mUsbDev == null)
            mTextViewResult.setText("No Device been selected");
        else
        {
            int count;
            count = mUsbDev.getInterfaceCount();
            String s = Integer.toHexString(mUsbDev.getProductId()) + "-" +
                    Integer.toHexString(mUsbDev.getVendorId()) + "\n"
                    + "Found "+ count + " usb interface";
            mTextViewResult.setText(s);
        }
    }



    @Override
    protected void   onDestroy() {
        Log.e(TAG,"onDestroy");
        unregisterReceiver(mReceiver);
        if (mMyDev != null)
        {
            try {
                mMyDev.Close();
            } catch (ReaderHwException e) {
                e.printStackTrace();
            }
        }
        super.onDestroy();
    }

    private static int closeReaderUp(ReaderActivity act){
        Log.d(TAG, "Closing reader...");
        int ret = 0;
        if (act.mReader != null)
        {
            ret = act.mReader.close();
        }
        return ret;
    }

    private void closeReaderBottom(ReaderActivity act){
        onCreateButtonSetup();
        act.mTextViewResult.setText("");
        try {
            act.mMyDev.Close();
        } catch (ReaderHwException e) {
            e.printStackTrace();
        }
    }




    public class CloseTask extends AsyncTask <Void, Void, Integer> {
        private final WeakReference<ReaderActivity> weakActivity;

        CloseTask(ReaderActivity myActivity) {
            this.weakActivity = new WeakReference<ReaderActivity>(myActivity);
        }

        @Override
        protected void onPreExecute() {
            ReaderActivity activity = weakActivity.get();
            setUpWaitDialog(activity,"Closing Reader");

        }
        @Override
        protected Integer doInBackground(Void... params) {
            int status = 0;
            ReaderActivity activity = weakActivity.get();
            try {
                do{
                    status = closeReaderUp(activity);
                }while(SCError.maskStatus(status) == SCError.READER_CMD_BUSY);

            } catch (Exception e) {
                activity.mStrMessage = "Get Exception : " + e.getMessage();
                //mTextViewResult.setText( mStrMessage);
                return -1;
            }

            return status;
        }

        @Override
        protected void onPostExecute(Integer result) {
            ReaderActivity activity = weakActivity.get();
            if (result != 0) {
                activity.mTextViewResult.setText("Close fail: "+ SCError.errorCode2String(result) +". " + activity.mStrMessage);
                Log.e(TAG, "Close fail: "+ SCError.errorCode2String(result));
            }else{
                activity.mTextViewResult.setText("Close successfully");
                Log.e(TAG,"Close successfully");
            }
            closeReaderBottom(activity);
            activity.mWaitProgress.dismiss();
        }
    }



    private int getSlotStatus(){
        int ret = SCError.READER_NO_CARD;
        byte []pCardStatus = new byte[1];

        /*detect card hotplug events*/

        ret = mReader.getCardStatus(pCardStatus);
        if (ret == SCError.READER_SUCCESSFUL){
            if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_ABSENT ){
                ret = SCError.READER_NO_CARD;
            }
            else if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_INACTIVE)
            {
                ret = SCError.READER_CARD_INACTIVE;
            }
            else
                ret = SCError.READER_SUCCESSFUL;
        }

        return ret;
    }

    public StringBuilder dbgMsg = new StringBuilder();
    private int InitReader(ReaderActivity act)
    {
        int Status = 0;
        boolean init;//
        Log.d(TAG, "InitReader");
        dbgMsg .append("InitReader \n");
        try {
            init = act.mMyDev.init(act.mManager, act.mUsbDev);
            if (!init){
                Log.e(TAG, "Device init fail");
                return -1;
            }
            mReader = readerSpecificInit();
        } catch (ReaderHwException e) {
            act.mStrMessage = "Get Exception : " + e.getMessage();
            return -1;
        } catch (ReaderException e) {
            act.mStrMessage = "Get Exception : " + e.getMessage();
            return -1;
        }


        return Status;
    }

    abstract Reader readerSpecificInit() throws ReaderException;

    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {

        public void onReceive(Context context, Intent intent) {
            Log.d(TAG, "Broadcast Receiver");

            String action = intent.getAction();
            if (ACTION_USB_PERMISSION.equals(action)) {
                synchronized (this) {
                    UsbDevice device = (UsbDevice) intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                    if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
                        if (device != null) {
                            onDevPermit(device);
                        }
                    } else {
                        Log.d(TAG, "Permission denied for device " + device.getDeviceName());
                    }
                }

            } else if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {

                Log.d(TAG, "Device Detached");
                onDetach(intent);

                synchronized (this) {
                }
            }
        }/*end of onReceive(Context context, Intent intent) {*/
    };




    public abstract void onCreateButtonSetup();


    public abstract void onOpenButtonSetup( );

    private void onDevPermit(UsbDevice dev){
        try {
            setUpWaitDialog(this,"Opening...");
            new OpenTask(this).execute(dev);
        }
        catch(Exception e){
            mStrMessage = "Get Exception : " + e.getMessage();
            Log.e(TAG, mStrMessage);
        }
    }


    private void onDetach(Intent intent){
        UsbDevice   udev = (UsbDevice) intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
        if (udev != null ) {
            if (udev.equals(mUsbDev) ){
                closeReaderUp(this);
                closeReaderBottom(this);
                this.finish();
            }
        }
        else {
            Log.d(TAG,"usb device is null");
        }
    }



    private  class OpenTask extends AsyncTask <UsbDevice, Void, Integer> {
        private final WeakReference<ReaderActivity> weakActivity;

        OpenTask(ReaderActivity myActivity) {
            this.weakActivity = new WeakReference<ReaderActivity>(myActivity);
        }

        @Override
        protected Integer doInBackground(UsbDevice... params) {
            int status = 0;
            ReaderActivity activity = weakActivity.get();
            status = InitReader(activity) ;
            if ( status != 0){
                Log.e(TAG, "fail to initial reader");
                return status;
            }
            return status;
        }

        @Override
        protected void onPostExecute(Integer result) {
            ReaderActivity activity = weakActivity.get();
            activity.mWaitProgress.dismiss();
            String s;
            if (result != 0) {
                 s = dbgMsg.toString() +"Open fail: " + activity.mStrMessage;

                activity.mTextViewResult.setText(s);
                Log.e(TAG,"Open fail: " + activity.mStrMessage);

            }else{
                s = dbgMsg.toString() +"Open successfully";
                activity.mTextViewResult.setText(s);
                Log.e(TAG,"Open successfully");
                onOpenButtonSetup();
            }

        }
    }

    public int poweron(){
        int result = SCError.READER_SUCCESSFUL;
        Log.d(TAG,"poweron");
        //check slot status first
        result = getSlotStatus();
        switch (result)
        {
            case SCError.READER_NO_CARD:
                return SCError.READER_NO_CARD;
            case SCError.READER_CARD_INACTIVE:
            case SCError.READER_SUCCESSFUL:
                break;
            default://returns other error case
                return result;
        }

        result = mReader.setPower(Reader.CCID_POWERON);

        Log.d(TAG,"power on exit");
        return result;
    }

    public int poweroff(){
        int result =  SCError.READER_SUCCESSFUL;
        Log.d(TAG,"poweroff");
        result = getSlotStatus();
        switch (result)
        {
            case SCError.READER_NO_CARD:
                return SCError.READER_NO_CARD;
            case SCError.READER_CARD_INACTIVE:
            case SCError.READER_SUCCESSFUL:
                break;
            default://returns other error case
                return result;
        }
        //----------poweroff card------------------
        result = mReader.setPower(Reader.CCID_POWEROFF);
        return result;
    }


    public void requestDevPerm(){
        if (mUsbDev != null)
            mManager.requestPermission(mUsbDev, mPermissionIntent);
        else
        {
            Log.e(TAG,"selected not found");
            mTextViewResult.setText("Device not found");
        }
    }

    public static void setUpWaitDialog(ReaderActivity act, String msg){

        act.mWaitProgress.setMessage(msg);
        act.mWaitProgress.setCancelable(false);
        act.mWaitProgress.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        act.mWaitProgress.show();
    }

    public void setupViews(){
        Log.d(TAG,"setupViews");
        setPowerView();
        setButtonView();
        setTextView();
        setEditView();
        onCreateButtonSetup();
        mWaitProgress =  new ProgressDialog(ReaderActivity.this);
        //setReaderSlotView();
    }

    abstract void setButtonView();
    abstract void setTextView();
    abstract void setEditView();

    public void setPowerView(){
        mPowerDialog = new AlertDialog.Builder(this);
        DialogInterface.OnClickListener poweroffClick = new DialogInterface.OnClickListener(){
            @Override
            public void onClick(DialogInterface dialog, int which) {
                int ret;
                ret =	poweroff();
                if (ret == SCError.READER_SUCCESSFUL)
                    mTextViewResult.setText("power off successfully");
                else if (SCError.maskStatus(ret) == SCError.READER_NO_CARD){
                    mTextViewResult.setText("Card Absent");
                }
                else
                {
                    showResult("PowerOff", ret);
                }
            }
        };

        DialogInterface.OnClickListener poweronClick = new DialogInterface.OnClickListener()
        {
            public void onClick(DialogInterface dialog, int which) {
                int ret;
                ret =	poweron();
                if (ret == SCError.READER_SUCCESSFUL)
                {
                    mTextViewResult.setText("power on successfully");
                }
                else if (SCError.maskStatus(ret) == SCError.READER_NO_CARD){
                    mTextViewResult.setText("Card Absent");
                }
                else
                {
                    showResult("PowerOn", ret);
                }
            }
        };
        mPowerDialog.setTitle("Set Power");
        mPowerDialog.setNeutralButton("PowerON", poweronClick);
        mPowerDialog.setPositiveButton("PowerOFF", poweroffClick);
    }


    public void showBufResult(int len, byte[]pBuf) {
        String s;
        s = MainActivity.byte2String(pBuf, len);
        mTextViewResult.setText(s);
    }

    public void showResult(String cmd, int result)
    {
        String  msg, ccidErrCode;
        if (result == SCError.READER_SUCCESSFUL)
        {
            mTextViewResult.setText(cmd + " Success");
            return;
        }

        msg = new String("Fail:"+SCError.errorCode2String(result)+ "("+Integer.toHexString(result)+")");

        if (SCError.maskStatus(result) == SCError.READER_CMD_FAIL)
        {
            ccidErrCode = Integer.toHexString(mReader.getCmdFailCode() & 0x0000ffff);
            msg = cmd + " " +msg + "(" + ccidErrCode +")";
        }
        mTextViewResult.setText(msg);
    }


    public void toRegisterReceiver(){
        // Register receiver for USB permission
        mPermissionIntent = PendingIntent.getBroadcast(this, 0, new Intent(ACTION_USB_PERMISSION), 0);
        IntentFilter filter = new IntentFilter();
        filter.addAction(ACTION_USB_PERMISSION);
        filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        registerReceiver(mReceiver, filter);
    }



    public interface OnReadyCallback {
        int deviceReady(int status) ;
    }

}
