package com.alcorlink.smartcard;


import amlib.ccid.SCError;
import amlib.ccid.Reader;
import amlib.ccid.Reader4442;
import amlib.ccid.ReaderException;
import amlib.hw.HWType;
import amlib.hw.HardwareInterface;
import amlib.hw.HardwareInterfaceUsb;
import amlib.hw.ReaderHwException;

import android.app.AlertDialog;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.app.AlertDialog.Builder;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.AsyncTask;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import java.lang.ref.WeakReference;

public class SLE4442_Activity extends ReaderActivity {

	private String mStrMessage;
	private Reader4442 m4442;

	private Button mBtOpen;
	private Button mBtClose;
	private Button mBtPower;
	private Button mBtVerify;
	private Button mBtReadMM;
	private Button mBtUpdateMM;
	private Button mBtReadPM;
	private Button mBtWritePM;
	private Button mBtReadSM;
	private Button mBtUpdataSM;
	private Button mBtCompare;

	
	private EditText mEdtVerifyPin1;
	private EditText mEdtVerifyPin2;
	private EditText mEdtVerifyPin3;
	private EditText mEdtDataMM;
	private EditText mEdtAddrMM;
	private EditText mEdtLenMM;
	private EditText mEdtAddrPM;
	private EditText mEdtDataPM;
	private EditText mEdtAddrSM;
	private EditText mEdtDataSM;
	


	private static final String TAG = MainActivity.TAG;
	private static final int DIGIT_OF_BYTE = 2;
	private static final byte FIXED_LEN_TO_READ = 4;

	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		Log.d(TAG," onCreate");
		setContentView(R.layout.activity_sle4442);
		setupViews();
		super.onCreate(savedInstanceState);
	}


	public void OnClickSLE4442Open(View view){
		Log.d(TAG, "4442OpenOnClick");
		requestDevPerm();
	}
	

	public void OnClickSLE4442Power(View view){
		mPowerDialog.show();
	}

	
	public void OnClickSLE4442Verify(View view){
		byte []pData = new byte[1];
		byte []pPin = new byte[3];
		int result;
		getEditTextHexByte(mEdtVerifyPin1, pData);
		pPin[0] = pData[0];
		getEditTextHexByte(mEdtVerifyPin2, pData);
		pPin[1] = pData[0];
		getEditTextHexByte(mEdtVerifyPin3, pData);
		pPin[2] = pData[0];
		
		result = m4442.SLE4442Cmd_Verify(3, pPin);
		showResult("Verify", result);
	}
	
	public void OnClickSLE4442ReadMainMemory(View view){
		Log.d(TAG, "SLE4442ReadMainMemoryOnClick");
		int result;
		byte[]pAddress = new byte[1];
		byte []pLen = new byte [1];
		byte []pReadData = null;
		int []pIntReturnLen = new int[1];
		getEditTextHexByte(mEdtAddrMM, pAddress);
		getEditTextHexByte(mEdtLenMM, pLen);

		Log.d("TAG", "pAddress="+Integer.toString(pAddress[0], 10));
		pReadData = new byte[pLen[0]];
		result = m4442.SLE4442Cmd_ReadMainMemory(pAddress[0], pLen[0], pReadData, pIntReturnLen) ;
		if (result == SCError.READER_SUCCESSFUL )
		{
			Log.d(TAG,"read len "+pIntReturnLen[0]);
			showTextResult(pIntReturnLen[0], pReadData);
		}
		else
			showResult("ReadMainMemory", result);
	}

	public void  OnClickSLE4442UpdataMainMemory(View view){
		Log.d(TAG, "SLE4442UpdataMainMemoryOnClick");
		byte[]pAddress = new byte[1];
		byte []pData = new byte[1];
		getEditTextHexByte(mEdtAddrMM, pAddress);
		getEditTextHexByte(mEdtDataMM, pData);
		showResult("UpdataMainMemory", m4442.SLE4442Cmd_UpdateMainMemory(pAddress[0], pData[0]));
	}
	
	public void  OnClickSLE4442ReadProtectionMemory(View view){
		Log.d(TAG, "SLE4442ReadProtectionMemoryOnClick");
		int result;
		byte[]pAddress = new byte[1];
		byte []pReadData = null;
		int []pIntReturnLen = new int[1];
		getEditTextHexByte(mEdtAddrPM, pAddress);
	
		pReadData = new byte[FIXED_LEN_TO_READ];
		result = m4442.SLE4442Cmd_ReadProtectionMemory(FIXED_LEN_TO_READ, pReadData, pIntReturnLen);
		if (result == SCError.READER_SUCCESSFUL )
		{
			Log.d(TAG,"read len "+pIntReturnLen[0]);
			showTextResult(pIntReturnLen[0], pReadData);
		}
		else
			showResult("ReadProtectionMemory", result);
	}

	public void  OnClickSLE4442WriteProtectionMemory(View view){
		Log.d(TAG, "SLE4442WriteProtectionMemoryOnClick");
		int result;
		byte[]pAddress = new byte[1];
		byte []pData = new byte[1];
		getEditTextHexByte(mEdtAddrPM, pAddress);
		getEditTextHexByte(mEdtDataPM, pData);
		result = m4442.SLE4442Cmd_WriteProtectionMemory(pAddress[0], pData[0]);
		
		showResult("WriteProtectionMemory", result);
	}

	public void  OnClickSLE4442ReadSecurityMemory(View view){
		Log.d(TAG, "SLE4442CReadSecurityMemoryOnClick");
		int result;
		byte[]pAddress = new byte[1];
		byte []pReadData = null;
		int []pIntReturnLen = new int[1];
		getEditTextHexByte(mEdtAddrSM, pAddress);
	
		pReadData = new byte[FIXED_LEN_TO_READ];
		result = m4442.SLE4442Cmd_ReadSecurityMemory( FIXED_LEN_TO_READ, pReadData, pIntReturnLen);
		if (result == SCError.READER_SUCCESSFUL )
		{
			Log.d(TAG,"read len "+pIntReturnLen[0]);
			showTextResult(pIntReturnLen[0], pReadData);
		}
		else
			showResult("ReadSecurityMemory", result);
	}

	public void  OnClickSLE4442UpdataSecurityMemory(View view){
		Log.d(TAG, "SLE4442UpdataSecurityMemoryOnClick");
		int result;
		byte[]pAddress = new byte[1];
		byte []pData = new byte[1];
		getEditTextHexByte(mEdtAddrSM, pAddress);
		getEditTextHexByte(mEdtDataSM, pData);
		result = m4442.SLE4442Cmd_UpdateSecurityMemory(pAddress[0], pData[0]);
		showResult("UpdataSecurityMemory", result);
	}

	public void  OnClickSLE4442ComparaeVerification(View view){
		Log.d(TAG, "SLE4442ComparaeVerificationOnClick");
		int result;
		byte[]pAddress = new byte[1];
		byte []pData = new byte[1];
		getEditTextHexByte(mEdtAddrSM, pAddress);
		getEditTextHexByte(mEdtDataSM, pData);
		result = m4442.SLE4442Cmd_CompareVerificationData(pAddress[0], pData[0]);
		showResult("ComparaeVerification", result);
	}

	
	public void OnClickSLE4442Close(View view){
		new CloseTask(SLE4442_Activity.this).execute();
	}





	@Override
	public Reader readerSpecificInit() throws ReaderException{
		int status;
		Log.d(TAG,"readerSpecificInit");

		m4442 = new Reader4442((HardwareInterface) mMyDev);
		status = m4442.open();
		if (status != SCError.READER_SUCCESSFUL) {
			String s = "open SLE4442 fail:" + SCError.errorCode2String(status);
			Log.e(TAG, s);
			throw new ReaderException(s);
		}
		return m4442;
	}

	
	/*
	 * 
	 * @param edt
	 * @param pBuf
	 * @param length number of digits
	 * @param pRetLen  how many bytes
	 * @return
	 */
	private int getData(EditText edt, byte []pBuf, int length, byte []pRetLen)
	{
		byte []pByteArrary;
		String strText; 
		int len;
		strText = edt.getText().toString();
		
		len = strText.length() ;
		Log.d(TAG, "Lenght="+ len);
		if (len > (length*2) || len == 0 || (len%2 != 0))
		{
			mStrMessage = "Wrong length in Data Fields ";
			Log.e(TAG, mStrMessage);
			mTextViewResult.setText(mStrMessage);
			return -1;
		}
		pByteArrary = MainActivity.toByteArray(strText);
		len = pByteArrary.length;
		
		Log.d(TAG, "Byte array Lenght="+ len);
	
		for (int i=0; i<(len); i++)
		{
			pBuf[i] = pByteArrary[i];
			Log.d(TAG, "pBuf[" + i +"]=0x"+Integer.toHexString(pBuf[i]));
		}
		 
		pRetLen[0] = (byte)len;
		return 0;
	}

	/*
	 * length: number of max digits in this field
	 * */ 
	private int getEditTextHexByte(EditText edt, byte []pData)
	{
		byte []pBuf = new byte[2];
		byte []pReturnLen = new byte[1];
		int result;
		result = getData(edt, pBuf, DIGIT_OF_BYTE, pReturnLen);
		if (result != 0)
			return result;
		if (pReturnLen[0] != 1) //one byte
			return -1;
		pData[0] = pBuf[0];
		Log.d(TAG, "Read Byte =" + Integer.toHexString(pBuf[0]));
		return 0;
	}

	
	private int getSlotStatus(){
		int ret = SCError.READER_NO_CARD;
		byte []pCardStatus = new byte[1];

		/*detect card hotplug events*/
		ret = m4442.getCardStatus(pCardStatus);
		if (ret == SCError.READER_SUCCESSFUL) {
			if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_ABSENT) {
				ret = SCError.READER_NO_CARD;
			} else if (pCardStatus[0] == Reader.SLOT_STATUS_CARD_INACTIVE) {
				ret = SCError.READER_CARD_INACTIVE;
			} else
				ret = SCError.READER_SUCCESSFUL;
		}
		return ret;
	}



	@Override
	public void onCreateButtonSetup(){
		if (mUsbDev != null)
			mBtOpen.setEnabled(true);
		else
			mBtOpen.setEnabled(false);
		mBtPower.setEnabled(false);
		mBtVerify.setEnabled(false);
		mBtReadMM.setEnabled(false);
		mBtUpdateMM.setEnabled(false);
		mBtReadPM.setEnabled(false);
		mBtWritePM.setEnabled(false);
		mBtReadSM.setEnabled(false);
		mBtUpdataSM.setEnabled(false);
		mBtCompare.setEnabled(false);
		mBtClose.setEnabled(false);
	}




	@Override
	public void onOpenButtonSetup() {
		mBtOpen.setEnabled(false);
		mBtPower.setEnabled(true);
		mBtVerify.setEnabled(true);
		mBtReadMM.setEnabled(true);
		mBtUpdateMM.setEnabled(true);
		mBtReadPM.setEnabled(true);
		mBtWritePM.setEnabled(true);
		mBtReadSM.setEnabled(true);
		mBtUpdataSM.setEnabled(true);
		mBtCompare.setEnabled(true);
		mBtClose.setEnabled(true);
	}



	@Override
	public void setButtonView()
	{
		mBtOpen = (Button)findViewById(R.id.SLE4442buttonOpen);
		mBtClose = (Button)findViewById(R.id.SLE4442buttonClose);
		mBtPower = (Button)findViewById(R.id.SLE4442buttonPower);
		mBtVerify = (Button)findViewById(R.id.SLE4442buttonVerify);
		mBtReadMM = (Button)findViewById(R.id.SLE4442buttonReadMM);
		mBtUpdateMM = (Button)findViewById(R.id.SLE4442buttonUpdataMM);
		mBtReadPM = (Button)findViewById(R.id.SLE4442buttonReadPM);
		mBtWritePM= (Button)findViewById(R.id.SLE4442buttonWritePM);
		mBtReadSM = (Button)findViewById(R.id.SLE4442buttonReadSM);
		mBtUpdataSM = (Button)findViewById(R.id.SLE4442buttonUpdataSM);
		mBtCompare = (Button)findViewById(R.id.SLE4442buttonCompare);
	}

	@Override
	public void setEditView()
	{
		mEdtVerifyPin1 = (EditText)findViewById(R.id.SLE4442EditTextPIN1);
		mEdtVerifyPin2 = (EditText)findViewById(R.id.SLE4442EditTextPIN2);
		mEdtVerifyPin3 = (EditText)findViewById(R.id.SLE4442EditTextPIN3);
		mEdtAddrMM = (EditText)findViewById(R.id.SLE4442EditTextAddrMM);
		mEdtLenMM = (EditText)findViewById(R.id.SLE4442EditTextLenMM);
		mEdtDataMM = (EditText)findViewById(R.id.SLE4442EditTextDataMM);
		mEdtAddrPM = (EditText)findViewById(R.id.SLE4442EditTextAddrPM);
		mEdtDataPM = (EditText)findViewById(R.id.SLE4442EditTextDataPM);
		mEdtAddrSM = (EditText)findViewById(R.id.SLE4442EditTextAddrSM);
		mEdtDataSM = (EditText)findViewById(R.id.SLE4442EditTextDataSM);
	}


	@Override
	public void setTextView()
	{
		mTextViewResult = (TextView)findViewById(R.id.SLE4442TextResult);  
	}

	private void showTextResult(int len, byte []pData)
	{
		int i;
		StringBuffer msg = new StringBuffer();
		msg.append("ReadData:");
		for (i=0;i< len;i++)
		{
			msg.append(" "+ Integer.toHexString(0x000000ff&pData[i]));
		}
		mTextViewResult.setText(msg.toString());
	}

}
